@@ -9,7 +9,7 @@
 #undef NO_RMEMSET
 
 /* Define if you don't want any resources read */
-#undef NO_RESOURCES 
+#undef NO_RESOURCES
 
 /* Define if you want to use XGetDefault instead of our internal version
  * which only reads ~/.Xdefaults, if it exists, otherwise ~/.Xresources if it
@@ -76,19 +76,19 @@
    or libasimage if you want to integrate with AfterStep */
 #undef BACKGROUND_IMAGE
 
-/* Define if you want to use libXpm for background imaging 
+/* Define if you want to use libXpm for background imaging
    previos one must be defined too */
 #undef XPM
 
-/* Define if you want to use libjpeg for background imaging 
+/* Define if you want to use libjpeg for background imaging
    requires: BACKGROUND_IMAGE,USE_LIBASIMAGE */
 #undef JPEG
 
-/* Define if you want to use libpng for background imaging 
+/* Define if you want to use libpng for background imaging
    requires: BACKGROUND_IMAGE,USE_LIBASIMAGE */
 #undef PNG
 
-/* Define if you want to integrate with AfterStep for background imaging 
+/* Define if you want to integrate with AfterStep for background imaging
    requires: BACKGROUND_IMAGE and any of XPM, JPEG, PNG */
 #undef USE_LIBASIMAGE
 #undef LIBASIMAGE_HEADERS
@@ -146,3 +146,10 @@
 #undef RUINT32T
 #undef RINTPT
 #undef RUINTPT
+
+/*
+ * Define if you need Thai language support
+ * Only display stuff
+ * Input method should be handles by either xim, xkb, or xmodmap
+ */
+#undef THAI
