@@ -7,17 +7,17 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *----------------------------------------------------------------------*/
-/*----------------------------------------------------------------------* 
+/*----------------------------------------------------------------------*
  * Originally written:
  *    1992      John Bovey, University of Kent at Canterbury <jdb@ukc.ac.uk>
  * Modifications:
@@ -42,12 +42,14 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: command.c,v 1.5 2000/05/28 07:46:35 sashav Exp $";
+static const char rcsid[] = "$Id: command.c,v 1.7 2001/09/06 15:57:04 sashav Exp $";
 #endif
 
 /*{{{ includes: */
 #include "rxvt.h"		/* NECESSARY */
 
+#define TT_PRINTF_LIMIT		1024
+
 #if defined(OFFIX_DND) || defined(TRANSPARENT)
 # include <X11/Xatom.h>
 #endif
@@ -480,8 +482,12 @@ get_pty(void)
 	extern char    *ptsname();
 
     /* open the STREAMS, clone device /dev/ptmx (master pty) */
+#ifdef HAVE_GETPT
+	if ((fd = getpt()) < 0) {
+#else
 	if ((fd = open("/dev/ptmx", O_RDWR)) < 0) {
-	    goto Failed;
+#endif
+		goto Failed;
 	} else {
 	    grantpt(fd);	/* change slave permissions */
 	    unlockpt(fd);	/* unlock slave */
@@ -1207,10 +1213,10 @@ init_xlocale(void)
     Input_Context = NULL;
 
 # ifndef NO_SETLOCALE
-    setlocale(LC_CTYPE, "");	/* XXX: should we do this? */
+   /* setlocale(LC_CTYPE, "");	/* XXX: should we do this? */
 # endif
     if (rs_inputMethod == NULL || !*rs_inputMethod) {
-	if ((p = XSetLocaleModifiers("@im=none")) != NULL && *p)
+	if ((p = XSetLocaleModifiers("")) != NULL && *p)
 	    xim = XOpenIM(Xdisplay, NULL, NULL, NULL);
     } else {
 	STRCPY(tmp, rs_inputMethod);
@@ -1238,7 +1244,7 @@ init_xlocale(void)
 	}
     }
 
-    if (xim == NULL && (p = XSetLocaleModifiers("")) != NULL && *p)
+	if (xim == NULL && (p = XSetLocaleModifiers("@im=none")) != NULL && *p)
 	xim = XOpenIM(Xdisplay, NULL, NULL, NULL);
 
     if (xim == NULL) {
@@ -1445,6 +1451,20 @@ lookup_key(XEvent * ev)
 		}
 		break;
 
+	    case XK_Up:		/* Shift+XK_Up = scroll up one line */
+		if (TermWin.saveLines) {
+		    scr_page(UP, 1);
+		    return;
+		}
+		break;
+
+	    case XK_Down:	/* Shift+XK_Down = scroll down one line */
+		if (TermWin.saveLines) {
+		    scr_page(DN, 1);
+		    return;
+		}
+		break;
+
 	    case XK_Next:	/* Shift+Next = scroll forward */
 		if (TermWin.saveLines) {
 		    scr_page(DN, lnsppg);
@@ -1576,11 +1596,11 @@ lookup_key(XEvent * ev)
 	    case XK_KP_Left:	/* \033Ot or standard */
 	    case XK_KP_Up:	/* \033Ox or standard */
 	    case XK_KP_Right:	/* \033Ov or standard */
-	    case XK_KP_Down:	/* \033Ow or standard */
+	    case XK_KP_Down:	/* \033Or or standard */
 		if ((PrivateModes & PrivMode_aplKP) ? !shft : shft) {
 		    len = 3;
 		    STRCPY(kbuf, "\033OZ");
-		    kbuf[2] = ("txvw"[keysym - XK_KP_Left]);
+		    kbuf[2] = ("txvr"[keysym - XK_KP_Left]);
 		    break;
 		} else {
 		/* translate to std. cursor key */
@@ -1747,7 +1767,7 @@ lookup_key(XEvent * ev)
     len = 5;								\
     sprintf((char *) kbuf,"\033[%02d~", (int)((n) + (keysym - fkey)))
 
-	    case XK_F1:		/* "\033[11~" */ 
+	    case XK_F1:		/* "\033[11~" */
 	    case XK_F2:		/* "\033[12~" */
 	    case XK_F3:		/* "\033[13~" */
 	    case XK_F4:		/* "\033[14~" */
@@ -2061,7 +2081,7 @@ mouse_report(XButtonEvent * ev)
 {
     int             button_number, key_state = 0;
     int             x, y;
-    
+
     x = ev->x;
     y = ev->y;
     pixel_position(&x, &y);
@@ -2218,7 +2238,7 @@ process_x_event(XEvent * ev)
 #ifdef OFF_FOCUS_FADING
 	    if( rs_fade != NULL )
 	    {
-		PixColors = &(PixColorsUnFocused[0]); 
+		PixColors = &(PixColorsUnFocused[0]);
 		on_colors_changed(Color_bg);
 	    }
 #endif
@@ -2245,25 +2265,25 @@ process_x_event(XEvent * ev)
     case SelectionRequest:
 	selection_send(&(ev->xselectionrequest));
 	break;
-	
+
      case PropertyNotify:
 #ifdef _MYSTYLE_
-	if (ev->xproperty.atom == _AS_STYLE ) 
+	if (ev->xproperty.atom == _AS_STYLE )
 	{
 	  const char *mystyle_name ;
 	  MyStyle *mystyle ;
 
-	    if( rs_mystyle ) 
+	    if( rs_mystyle )
 		mystyle_name = rs_mystyle ;
-	    else 
+	    else
 		mystyle_name = GetDefaultMyStyle() ;
 
 	    mystyle_get_property(Xdisplay, Xroot, _AS_STYLE, XA_INTEGER );
-	    
+
 	    set_mystyle( mystyle_find( mystyle_name ), True );
-	    break;	    
+	    break;
 	}
-	else if( TermWin.background.trgType == BGT_MyStyle && 
+	else if( TermWin.background.trgType == BGT_MyStyle &&
 		 TermWin.background.mystyle->texture_type < TEXTURE_TRANSPARENT)
 	    break ;
 #endif
@@ -2277,12 +2297,12 @@ process_x_event(XEvent * ev)
 
 	    static Atom atom = None ;
 	    if( atom == None ) atom = XInternAtom(Xdisplay, "_XROOTPMAP_ID", True);
-	    
+
 	    if( !atom )
 	    {
 		SetSrcPixmap(None);
 	    }else
-	        if (ev->xproperty.atom == atom) 
+	        if (ev->xproperty.atom == atom)
 		{
 		  static Pixmap LastRPixmap = None;
 		  Pixmap currentRPixmap = GetRootPixmap(atom);
@@ -2297,7 +2317,7 @@ process_x_event(XEvent * ev)
 		        	SetSrcPixmap(currentRPixmap);
 		        	RenderPixmap(1);
 				/* don't want to check if pixmap is valid
-				   it's unlikely that user would set it to 
+				   it's unlikely that user would set it to
 				   something bogus. */
 		    	    }
 		    	    scr_clear();
@@ -2317,14 +2337,14 @@ process_x_event(XEvent * ev)
 #endif /* TRANSPARENT */
 	break;
      case UnmapNotify:
-        TermWin.bMapped = 0 ; 
+        TermWin.bMapped = 0 ;
 /*	fprintf(stderr, "\n aterm is UnMapped(event)");*/
 	break;
 
      case MapNotify:
-/*        { 
+/*        {
 	  XWindowAttributes attr ;
-    	      
+
 	    XGetWindowAttributes( Xdisplay, ParentWin[0], &attr );
 	    TermWin.bMapped = (attr.map_state == IsViewable)?1:0 ;
 
@@ -2334,10 +2354,10 @@ process_x_event(XEvent * ev)
 	TermWin.bMapped = 1 ;
 #ifdef TRANSPARENT
 	refresh_transparent_scrollbar();
-	if( Options & Opt_transparent)	
+	if( Options & Opt_transparent)
 	{
 	  Pixmap tmp = GetRootPixmap(None);
-	    if( tmp != TermWin.background.srcPixmap && TermWin.background.srcPixmap != None )    
+	    if( tmp != TermWin.background.srcPixmap && TermWin.background.srcPixmap != None )
 	    {
 	        if( TermWin.background.trgType != BGT_None )
 	        {
@@ -2348,7 +2368,7 @@ process_x_event(XEvent * ev)
 	        scr_touch();
 	    }
 	}
-#endif	
+#endif
 	break ;
      case ReparentNotify:
 #ifdef TRANSPARENT
@@ -2356,8 +2376,8 @@ process_x_event(XEvent * ev)
 	    int n;
 	    XWindowAttributes attr ;
             int (*oldXErrorHandler) (Display *, XErrorEvent *) =
-		  XSetErrorHandler (pixmap_error_handler);	
-	    
+		  XSetErrorHandler (pixmap_error_handler);
+
 	    for( n = 1 ; n < PARENTS_NUM ; n ++ )    ParentWin[n] = None;
 	    /*
 	     * Make the frame window set by the window manager have
@@ -2377,12 +2397,12 @@ process_x_event(XEvent * ev)
 		/* we want to get all map/unmap events from this window as well*/
 	/*	XSelectInput(Xdisplay, parent, StructureNotifyMask); */
 
-		if (Options & Opt_transparent)     
+		if (Options & Opt_transparent)
 		    XSetWindowBackgroundPixmap(Xdisplay, parent, ParentRelative);
 	    }
 	    if( XGetWindowAttributes( Xdisplay, ParentWin[0], &attr ))
 		TermWin.bMapped = (attr.map_state == IsViewable)?1:0 ;
-	    
+
 	    XSetErrorHandler (oldXErrorHandler);
     	    scr_clear();
 	}
@@ -2714,7 +2734,7 @@ process_x_event(XEvent * ev)
 void
 tt_printf(const unsigned char *fmt,...)
 {
-    static unsigned char buf[256];
+	static unsigned char buf[TT_PRINTF_LIMIT];
     va_list         arg_ptr;
 
     va_start(arg_ptr, fmt);
@@ -2766,9 +2786,9 @@ process_print_pipe(void)
     if ((fd = popen_printer()) == NULL)
 	return;
 
-/* 
- * Send all input to the printer until either ESC[4i or ESC[?4i 
- * is received. 
+/*
+ * Send all input to the printer until either ESC[4i or ESC[?4i
+ * is received.
  */
     for (done = 0; !done;) {
 	unsigned char   buf[8];
@@ -3031,7 +3051,8 @@ process_csi_seq(void)
 	    break;
 #if defined (ENABLE_DISPLAY_ANSWER)
 	case 7:
-	    tt_printf((unsigned char *) "%s\n", display_name);
+		if( strlen(display_name)  < TT_PRINTF_LIMIT-2 )
+	    	tt_printf((unsigned char *) "%s\n", display_name);
 	    break;
 #endif
 	case 8:
@@ -3050,6 +3071,9 @@ process_csi_seq(void)
     /* drop */
     case 's':
     case 't':
+		if(arg[0] == 21)
+			tt_printf((unsigned char *) "\033]l%s\033\\", rs_title);
+        break;
     case 'h':
     case 'l':
 	process_terminal_mode(ch, priv, nargs, arg);
