@@ -1,5 +1,5 @@
 /*--------------------------------*-C-*---------------------------------*
- * File:	pixmap.c 
+ * File:	pixmap.c
  *----------------------------------------------------------------------*
  * Copyright (c) 1999 Ethan Fischer <allanon@crystaltokyo.com>
  * Copyright (c) 1999 Sasha Vasko   <sashav@sprintmail.com>
@@ -24,7 +24,7 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: pixmap.c,v 1.10 2000/07/01 02:22:19 sashav Exp $";
+static const char rcsid[] = "$Id: pixmap.c,v 1.3 2001/09/06 15:57:05 sashav Exp $";
 #endif
 
 #include "rxvt.h"		/* NECESSARY */
@@ -36,14 +36,19 @@ static const char rcsid[] = "$Id: pixmap.c,v 1.10 2000/07/01 02:22:19 sashav Exp
 # endif
 #endif
 
+/*#define BENCHMARK_SHADING 200 */
 /*#define DO_CLOCKING       */
 /*#define GETPIXEL_PUTPIXEL */
 /*#define DEBUG_IMAGING     */
 
-#ifdef DO_CLOCKING      
+#if defined(DO_CLOCKING) || defined(BENCHMARK_SHADING)
  #include <time.h>
 #endif
 
+#ifdef DEBUG_X
+#define XGetGeometry(dpy,win,r,x,y,w,h,b,d) \
+	trace_XGetGeometry(__FILE__,__LINE__,dpy,win,r,x,y,w,h,b,d)
+#endif
 
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
 /* Xdisplay, Xroot, Xvisual and Xdepth needs to be defined if code is moved to some other prog */
@@ -59,14 +64,14 @@ pixmap_error_handler (Display * dpy, XErrorEvent * error)
 #ifdef DEBUG_IMAGING
   fprintf(stderr, "\n aterm caused XError # %u, in resource %lu, Request: %d.%d",
           error->error_code, error->resourceid, error->request_code, error->minor_code );
-#endif	  
+#endif
   return 0;
 }
 
-#ifndef LIBASIMAGE_HEADERS
-void CopyAndShadeArea( Drawable src, Pixmap trg, 
-                       int x, int y, int w, int h, 
-		       int trg_x, int trg_y, 
+#ifndef USE_LIBASIMAGE /* used to be LIBASIMAGE_HEADERS*/
+void CopyAndShadeArea( Drawable src, Pixmap trg,
+                       int x, int y, int w, int h,
+		       int trg_x, int trg_y,
 		       GC gc, ShadingInfo* shading )
 {
   int (*oldXErrorHandler) (Display *, XErrorEvent *) ;
@@ -82,14 +87,29 @@ void CopyAndShadeArea( Drawable src, Pixmap trg,
         {
     	    ShadeXImage( img, shading, gc );
     	    XPutImage(Xdisplay, trg, gc, img, 0, 0, trg_x, trg_y, w, h);
+#ifdef BENCHMARK_SHADING
+	    {
+		int i;
+		time_t before, after;
+		double diff;
+
+		before = time (NULL);
+		for (i = 0; i < BENCHMARK_SHADING; i++)
+		    ShadeXImage (img, shading, gc);
+		after = time (NULL);
+
+		diff = difftime (after, before);
+		printf ("CopyAndShadeArea(): %d shading runs took %.0f seconds\n", BENCHMARK_SHADING, diff);
+	    }
+#endif
 	    XDestroyImage( img );
 	    return ;
 	}
     }
-    if( !XCopyArea (Xdisplay, src, trg, gc, x, y, w, h, trg_x, trg_y)) 
-	XFillRectangle( Xdisplay, trg, gc, trg_x, trg_y, w, h );			
+    if( !XCopyArea (Xdisplay, src, trg, gc, x, y, w, h, trg_x, trg_y))
+	XFillRectangle( Xdisplay, trg, gc, trg_x, trg_y, w, h );
     XSetErrorHandler (oldXErrorHandler);
-}    
+}
 
 int
 FillPixmapWithTile (Pixmap pixmap, Pixmap tile, int x, int y, int width,
@@ -120,11 +140,11 @@ FillPixmapWithTile (Pixmap pixmap, Pixmap tile, int x, int y, int width,
 void ShadeTiledPixmap(Pixmap src, Pixmap trg, int src_w, int src_h, int x, int y, int w, int h, GC gc, ShadingInfo* shading)
 {
   int tile_x, tile_y, left_w, bott_h;
-    
+
     tile_x = x%src_w ;
     tile_y = y%src_h ;
     left_w = min(src_w-tile_x,w);
-    bott_h = min(src_h-tile_y,h);    
+    bott_h = min(src_h-tile_y,h);
 /*fprintf( stderr, "\nShadeTiledPixmap(): tile_x = %d, tile_y = %d, left_w = %d, bott_h = %d, SRC = %dx%d TRG=%dx%d", tile_x, tile_y, left_w, bott_h, src_w, src_h, w, h);*/
     CopyAndShadeArea( src, trg, tile_x, tile_y, left_w, bott_h, 0, 0, gc, shading );
     if( bott_h < h )
@@ -139,7 +159,7 @@ void ShadeTiledPixmap(Pixmap src, Pixmap trg, int src_w, int src_h, int x, int y
     }
 }
 
-Pixmap 
+Pixmap
 ShadePixmap (Pixmap src, int x, int y, int width, int height, GC gc, ShadingInfo* shading )
 {
   Pixmap trg = CREATE_TRG_PIXMAP(width, height);
@@ -150,7 +170,7 @@ ShadePixmap (Pixmap src, int x, int y, int width, int height, GC gc, ShadingInfo
     return trg ;
 }
 
-Pixmap 
+Pixmap
 ScalePixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, ShadingInfo*  shading )
 {
   XImage *srcImage, *trgImage;
@@ -166,6 +186,21 @@ ScalePixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, Sha
 		{
 		    if( shading )     ShadeXImage( trgImage, shading, gc );
 		    XPutImage (Xdisplay, trg, gc, trgImage, 0, 0, 0, 0, width, height);
+#ifdef BENCHMARK_SHADING
+	    {
+		int i;
+		time_t before, after;
+		double diff;
+
+		before = time (NULL);
+		for (i = 0; i < BENCHMARK_SHADING; i++)
+		    ShadeXImage (trgImage, shading, gc);
+		after = time (NULL);
+
+		diff = difftime (after, before);
+		printf ("ScalePixmap(): %d shading runs took %.0f seconds\n", BENCHMARK_SHADING, diff);
+	    }
+#endif
 		}
 	        XDestroyImage (trgImage);
 	    }
@@ -175,7 +210,7 @@ ScalePixmap (Pixmap src, int src_w, int src_h, int width, int height, GC gc, Sha
     return trg;
 }
 
-Pixmap 
+Pixmap
 CenterPixmap( Pixmap src, int src_w, int src_h, int width, int height, GC gc, ShadingInfo*  shading)
 {
   int x, y, w, h, src_x = 0, src_y = 0 ;
@@ -187,28 +222,28 @@ CenterPixmap( Pixmap src, int src_w, int src_h, int width, int height, GC gc, Sh
 	/* fill it with background color */
         XFillRectangle(Xdisplay, trg, gc, 0, 0, width, height);
 	/* place image at the center of it */
-        x = (width-src_w)>>1 ;		    
+        x = (width-src_w)>>1 ;
 	y = (height-src_h)>>1 ;
-        if( x < 0 ) 
-	{ 
-	    src_x -= x ; 
-	    w = min(width,src_w+x) ; 
-	    x = 0 ; 
-	}else w = min(width, src_w) ; 
-        if( y < 0 ) 
-	{ 
-	    src_y -= y ; 
+        if( x < 0 )
+	{
+	    src_x -= x ;
+	    w = min(width,src_w+x) ;
+	    x = 0 ;
+	}else w = min(width, src_w) ;
+        if( y < 0 )
+	{
+	    src_y -= y ;
     	    h = min(height, src_h+y);
-	    y = 0 ; 
+	    y = 0 ;
 	}else h = min(height, src_h);
 
-	CopyAndShadeArea( src, trg, src_x, src_y, w, h, x, y, gc, shading);			
+	CopyAndShadeArea( src, trg, src_x, src_y, w, h, x, y, gc, shading);
     }
 
     return trg ;
 }
 
-Pixmap 
+Pixmap
 GrowPixmap( Pixmap src, int src_w, int src_h, int width, int height, GC gc, ShadingInfo*  shading)
 {
   int w, h;
@@ -220,23 +255,23 @@ GrowPixmap( Pixmap src, int src_w, int src_h, int width, int height, GC gc, Shad
 	/* fill it with background color */
         XFillRectangle(Xdisplay, trg, gc, 0, 0, width, height);
 	/* place image at the center of it */
-        w = min(width, src_w) ; 
+        w = min(width, src_w) ;
         h = min(height, src_h);
 
-	CopyAndShadeArea( src, trg, 0, 0, w, h, 0, 0, gc, shading);			
+	CopyAndShadeArea( src, trg, 0, 0, w, h, 0, 0, gc, shading);
     }
 
     return trg ;
 }
 /* PROTO */
-int 
+int
 GetRootDimensions( int* width, int* height )
 {
   Window root ;
   int w_x, w_y ;
-  unsigned int w_w, w_h, border_w, w_depth ; 
-    if( !XGetGeometry( Xdisplay, Xroot, &root, 
-	                  &w_x, &w_y, width, height, &border_w, &w_depth ))	
+  unsigned int w_w, w_h, border_w, w_depth ;
+    if( !XGetGeometry( Xdisplay, Xroot, &root,
+	                  &w_x, &w_y, width, height, &border_w, &w_depth ))
     {
 	*width = 0 ;
 	*height = 0 ;
@@ -244,7 +279,7 @@ GetRootDimensions( int* width, int* height )
     return (*width>0 && *height>0)?1:0 ;
 }
 
-int 
+int
 GetWinPosition (Window win, int *x, int *y)
 {
   Window root, parent, *children ;
@@ -260,36 +295,36 @@ GetWinPosition (Window win, int *x, int *y)
     if( !x ) x = &my_x ;
     if( !y ) y = &my_y ;
 
-    *x = 0 ; 
+    *x = 0 ;
     *y = 0 ;
-    
-    if( !rootWidth || !rootHeight ) 
+
+    if( !rootWidth || !rootHeight )
         if( !GetRootDimensions(&rootWidth, &rootHeight)) return 0;
 
     XGetGeometry( Xdisplay, win, &root, &dumm, &dumm, &w, &h, &udumm, &udumm );
-  
+
     while( XQueryTree( Xdisplay, win, &root, &parent, &children, &nchildren ) )
     {
       int w_x, w_y ;
-      unsigned int border_w ; 
+      unsigned int border_w ;
 	if( children ) XFree( children );
-	if( !XGetGeometry( Xdisplay, win, &root, &w_x, &w_y, &udumm, &udumm, &border_w, &udumm ))	 
+	if( !XGetGeometry( Xdisplay, win, &root, &w_x, &w_y, &udumm, &udumm, &border_w, &udumm ))
 	    break ;
 	(*x)+=w_x+(int)border_w ;
-	(*y)+=w_y+(int)border_w ;	
-	
+	(*y)+=w_y+(int)border_w ;
+
 	if( parent == root )
 	{/* taking in to consideration virtual desktopping */
 	  int bRes = 1 ;
 	    if( *x>=rootWidth  || (*x+(int)w) <= 0 ||
-	        *y>=rootHeight || (*y+(int)h) <= 0 ) 
+	        *y>=rootHeight || (*y+(int)h) <= 0 )
 		bRes = 0 ;
     /* don't want to return position outside the screen even if we fail */
     /*	    while( *x < 0 ) *x += rootWidth ;
 	    while( *y < 0 ) *y += rootHeight ;
 	    while( *x+(int)w > rootWidth ) *x -= rootWidth ;
 	    while( *y+(int)h > rootHeight) *y -= rootHeight ;
-     */	    return bRes; 
+     */	    return bRes;
 	}
 	win = parent ;
     }
@@ -315,16 +350,16 @@ sleep_a_little (int n)
 
 static Pixmap
 CutPixmap ( Pixmap src, Pixmap trg,
-            int x, int y, 
-	    unsigned int src_w, unsigned int src_h, 
-	    unsigned int width, unsigned int height, 
+            int x, int y,
+	    unsigned int src_w, unsigned int src_h,
+	    unsigned int width, unsigned int height,
 	    GC gc, ShadingInfo * shading)
 {
   Bool my_pixmap = (trg == None )?True:False ;
   int screen_w, screen_h ;
   int offset_x = 0, offset_y = 0 ;
-  
-  screen_w = DisplayWidth( Xdisplay, Xscreen );   
+
+  screen_w = DisplayWidth( Xdisplay, Xscreen );
   screen_h = DisplayHeight( Xdisplay, Xscreen );
 
   while( x+(int)width < 0 )  x+= screen_w ;
@@ -334,21 +369,21 @@ CutPixmap ( Pixmap src, Pixmap trg,
 
   if (width < 2 || height < 2 )
     return trg;
-  if( x < 0 )    
+  if( x < 0 )
   {
     offset_x = (-x) ;
-    x = 0 ; 
+    x = 0 ;
     width -= offset_x ;
   }
-  if( y < 0 )    
+  if( y < 0 )
   {
     offset_y = (-y) ;
     y = 0 ;
     height -= offset_y ;
   }
   if( x+width >= screen_w ) width = screen_w - x ;
-  if( y+height >= screen_h ) height = screen_h - y ;    
-  
+  if( y+height >= screen_h ) height = screen_h - y ;
+
   if (src == None) /* we don't have root pixmap ID */
     { /* we want to create Overrideredirect window overlapping out window
          with background type of Parent Relative and then grab it */
@@ -360,22 +395,22 @@ CutPixmap ( Pixmap src, Pixmap trg,
 	attr.backing_store = Always ;
 	attr.event_mask = ExposureMask ;
 	attr.override_redirect = True ;
-	src = XCreateWindow(Xdisplay, Xroot, x, y, width, height, 
-	                    0, 
+	src = XCreateWindow(Xdisplay, Xroot, x, y, width, height,
+	                    0,
 			    CopyFromParent, CopyFromParent, CopyFromParent,
 			    CWBackPixmap|CWBackingStore|CWOverrideRedirect|CWEventMask,
 			    &attr);
-        
+
 	if( src == None ) return trg ;
 	XGrabServer( Xdisplay );
 	grabbed = True ;
 	XMapRaised( Xdisplay, src );
 	XSync(Xdisplay, False );
-	/* now we have to wait for our window to become mapped - waiting for Expose */			    
+	/* now we have to wait for our window to become mapped - waiting for Expose */
 	for( tick_count = 0 ; !XCheckWindowEvent( Xdisplay, src, ExposureMask, &event ) && tick_count < 100 ; tick_count++)
 	    sleep_a_little(100);
-	    
-	if( tick_count < 100 )	
+
+	if( tick_count < 100 )
 	{
 	    if( trg == None )    trg = CREATE_TRG_PIXMAP (width+offset_x, height+offset_y);
 	    if (trg != None)
@@ -392,10 +427,25 @@ CutPixmap ( Pixmap src, Pixmap trg,
 		  {
     		    ShadeXImage (img, shading, gc);
 		    XPutImage (Xdisplay, trg, gc, img, 0, 0, offset_x, offset_y, width, height);
+#ifdef BENCHMARK_SHADING
+	    {
+		int i;
+		time_t before, after;
+		double diff;
+
+		before = time (NULL);
+		for (i = 0; i < BENCHMARK_SHADING; i++)
+		    ShadeXImage (img, shading, gc);
+		after = time (NULL);
+
+		diff = difftime (after, before);
+		printf ("CutPixmap(): %d shading runs took %.0f seconds\n", BENCHMARK_SHADING, diff);
+	    }
+#endif
 		    XDestroyImage (img);
 		  }else if( my_pixmap )
 		  {
-		    
+
 		    XFreePixmap( Xdisplay, trg );
 		    trg = None ;
 		  }
@@ -403,27 +453,27 @@ CutPixmap ( Pixmap src, Pixmap trg,
 		    XCopyArea (Xdisplay, src, trg, gc, 0, 0, width, height, offset_x, offset_y);
 	    }
         }
-	    
-	if( src ) 
+
+	if( src )
 	    XDestroyWindow( Xdisplay, src );
 	if( grabbed )
 	    XUngrabServer( Xdisplay );
 	return trg ;
     }
-  /* we have root pixmap ID */    
+  /* we have root pixmap ID */
   /* find out our coordinates relative to the root window */
   if (x + width > src_w || y + height > src_h)
     {			/* tiled pixmap processing here */
       Pixmap tmp ;
       width = min (width, src_w);
       height = min (height, src_h);
-      
+
       tmp = CREATE_TRG_PIXMAP (width, height);
       if (tmp != None)
       {
         ShadeTiledPixmap (src, tmp, src_w, src_h, x, y, width,
 			  height, gc, shading);
-        if( trg == None )    
+        if( trg == None )
            trg = CREATE_TRG_PIXMAP (width+offset_x, height+offset_y);
 	if( trg != None )
 	    XCopyArea (Xdisplay, tmp, trg, gc, 0, 0, width, height, offset_x, offset_y);
@@ -454,14 +504,14 @@ CutWinPixmap (Window win, Drawable src, int src_w, int src_h, int width,
 
   if( src == None )
   {
-#ifndef TRANSPARENT  
+#ifndef TRANSPARENT
     return None ;
 #else
-    if( !(Options & Opt_transparent)) 
+    if( !(Options & Opt_transparent))
 	return None ;
 #endif
   }
-  
+
   if (!GetWinPosition (win, &x, &y))
 	return None;
 
@@ -470,12 +520,12 @@ CutWinPixmap (Window win, Drawable src, int src_w, int src_h, int width,
 
 
 /* PROTO */
-Pixmap 
+Pixmap
 GetRootPixmap(Atom id)
 {
   Pixmap currentRootPixmap = None;
     if( id == None ) id = XInternAtom (Xdisplay, "_XROOTPMAP_ID", True);
-    
+
     if( id != None )
     {
       Atom act_type;
@@ -497,7 +547,7 @@ GetRootPixmap(Atom id)
 	}
     }
     return currentRootPixmap ;
-}    
+}
 
 
 Pixmap
@@ -513,7 +563,7 @@ ValidatePixmap(Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidth
     if(bTransparent)        p = GetRootPixmap( None );
     if( !pWidth ) pWidth = &junk ;
     if( !pHeight ) pHeight = &junk ;
-	
+
     if( p != None )
     {
         if (!XGetGeometry (Xdisplay, p, &root, &junk, &junk, pWidth, pHeight, &junk, &junk))
@@ -521,7 +571,7 @@ ValidatePixmap(Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidth
     }
     if( bSetHandler )
         XSetErrorHandler (oldXErrorHandler);
-    
+
     return p ;
 }
 
@@ -533,7 +583,7 @@ ValidatePixmap(Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidth
 #define BG TermWin.background /*for convinience*/
 
 /* PROTO */
-int 
+int
 GetMyPosition( int* x, int* y )
 {
   int bRet = 0 ;
@@ -541,7 +591,7 @@ GetMyPosition( int* x, int* y )
 
     bRet = GetWinPosition( TermWin.vt, x, y );
     XSetErrorHandler (old);
-  
+
     return bRet ;
 }
 
@@ -557,7 +607,7 @@ void FreeTargetPixmap()
 
 
 /* PROTO */
-Pixmap 
+Pixmap
 SetSrcPixmap(Pixmap p)
 {
     if( BG.srcPixmap != None && BG.bMySource )
@@ -567,10 +617,10 @@ SetSrcPixmap(Pixmap p)
 	BG.bMySource = 0 ;
     }
 
-    BG.srcPixmap = p ;    
+    BG.srcPixmap = p ;
 
     BG.Width = 0 ;
-    BG.Height = 0 ;	
+    BG.Height = 0 ;
 
     if( BG.srcPixmap != None )
     {
@@ -582,33 +632,33 @@ SetSrcPixmap(Pixmap p)
 			&w, &h, &dum, &dum))
 	{
 	    BG.Width = w ;
-	    BG.Height = h ;	
+	    BG.Height = h ;
 	}
     }
 }
 
 /* PROTO */
-void 
+void
 ValidateSrcPixmap(int bSetHandler)
 {
 
     if( !BG.bMySource )
     {    /* we need to check if pixmap is still valid */
       Pixmap new_p ;
-	
+
 	new_p = ValidatePixmap( BG.srcPixmap , bSetHandler,
-		     	        ((Options & Opt_transparent) && 
+		     	        ((Options & Opt_transparent) &&
 			          BG.trgType != BGT_None),
-				NULL, NULL );  
+				NULL, NULL );
 
 	if( new_p != BG.srcPixmap )	SetSrcPixmap(new_p);
     }
 }
 
 #ifdef _MYSTYLE_
-Pixmap 
-RenderMyStylePixmap( MyStyle *style, Pixmap root_pmap, 
-		     unsigned int root_pmap_width, unsigned int root_pmap_height, 
+Pixmap
+RenderMyStylePixmap( MyStyle *style, Pixmap root_pmap,
+		     unsigned int root_pmap_width, unsigned int root_pmap_height,
 		     unsigned int width, unsigned int height )
 {
 /*  fprintf( stderr, "Entering RenderMyStylePixmap : texture_type = %d\n", style->texture_type );
@@ -641,23 +691,23 @@ RenderPixmap(int DontCheckSource )
   unsigned int    height = TermWin_TotalHeight();
   unsigned int    fin_width, fin_height ;
   int (*oldXErrorHandler) (Display *, XErrorEvent *) ;
-    
+
     /* for convinience only */
     fin_width = width ;
     fin_height = height ;
-/*fprintf(stderr, "\n aterm: entering RenderPixmap, window size is %dx%d, trg_type = %d", width, height, BG.trgType );    
+/*fprintf(stderr, "\n aterm: entering RenderPixmap, window size is %dx%d, trg_type = %d", width, height, BG.trgType );
 */
     gcvalue.foreground = PixColors[Color_bg];
     gc = XCreateGC(Xdisplay, TermWin.vt, GCForeground, &gcvalue);
-    
+
     /* we have some nice processing of all the X errors built in */
     /* so let's not let us crash if anything goes wrong          */
     oldXErrorHandler = XSetErrorHandler (pixmap_error_handler);
-    
+
     if( !DontCheckSource )  ValidateSrcPixmap( 0 );
-    
-    
-    if( BG.srcPixmap == None 
+
+
+    if( BG.srcPixmap == None
 #ifdef _MYSTYLE_
 	&& BG.trgType != BGT_MyStyle
 #endif
@@ -665,20 +715,20 @@ RenderPixmap(int DontCheckSource )
     {
 #ifdef TRANSPARENT
 	if(!(Options & Opt_transparent) || BG.trgType == BGT_None)
-#endif    
+#endif
 	{
 	    XSetErrorHandler (oldXErrorHandler);
 	    return ; /* nothing to do here */
 	}
     }
-    
-/*fprintf(stderr, "\n aterm RenderPixmap(): freeing target pixmap ..."); 
+
+/*fprintf(stderr, "\n aterm RenderPixmap(): freeing target pixmap ...");
 */
-    if( BG.trgPixmap != BG.srcPixmap )	FreeTargetPixmap();		
-    
+    if( BG.trgPixmap != BG.srcPixmap )	FreeTargetPixmap();
+
 
     switch( BG.trgType )
-    { 
+    {
 #define SHADING ((BG.bMySource)?NULL:&(BG.Shading))
 	case BGT_Tile : /* just copying source PixampID into trgPixmapID */
 		if( BG.bMySource || NO_NEED_TO_SHADE(BG.Shading))
@@ -694,81 +744,81 @@ RenderPixmap(int DontCheckSource )
 		    BG.trgPixmap = ShadePixmap(BG.srcPixmap, 0, 0, fin_width, fin_height, gc, SHADING);
 		}
 		break ;
-	case BGT_Center : /* more complicated case */ 
+	case BGT_Center : /* more complicated case */
 		if( BG.finWidth != width || BG.finHeight!=height )
 		{
-/*fprintf(stderr, "\n aterm RenderPixmap(): (BGT_Center)src pixmap is [%lu]",BG.srcPixmap);	*/	
-		    BG.trgPixmap = CenterPixmap( BG.srcPixmap, 
-						 BG.Width, BG.Height, 
+/*fprintf(stderr, "\n aterm RenderPixmap(): (BGT_Center)src pixmap is [%lu]",BG.srcPixmap);	*/
+		    BG.trgPixmap = CenterPixmap( BG.srcPixmap,
+						 BG.Width, BG.Height,
 		                                 width, height, gc, SHADING );
 		}
-		break ;	
-	case BGT_Scale : 
+		break ;
+	case BGT_Scale :
 		if( BG.finWidth != width || BG.finHeight!=height )
 		{
 /*fprintf(stderr, "\n aterm RenderPixmap(): (BGT_Scale) src pixmap is [%lu]",BG.srcPixmap);*/
-	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap, 
-						BG.Width, BG.Height, 
+	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap,
+						BG.Width, BG.Height,
 		                                width, height, gc, SHADING );
 		}
-		break ;	
-	case BGT_ScaleH : 
+		break ;
+	case BGT_ScaleH :
 		if( BG.finWidth != width )
 		{
-	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap, 
-						BG.Width, BG.Height, 
+	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap,
+						BG.Width, BG.Height,
 		                                width, BG.Height, gc, SHADING );
 		    fin_height = BG.Height;
 		}
-		break ;	
-	case BGT_ScaleV : 
+		break ;
+	case BGT_ScaleV :
 		if( BG.finHeight!=height )
 		{
-	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap, 
-						BG.Width, BG.Height, 
+	    	    BG.trgPixmap = ScalePixmap( BG.srcPixmap,
+						BG.Width, BG.Height,
 		                                BG.Width, height, gc, SHADING );
-		    fin_width = BG.Width ; 
+		    fin_width = BG.Width ;
 		}
-		break ;	 
-	case BGT_NoTile : 
+		break ;
+	case BGT_NoTile :
 		if( BG.finWidth != width || BG.finHeight!=height )
 		{
-	    	    BG.trgPixmap = GrowPixmap(  BG.srcPixmap, 
-						BG.Width, BG.Height, 
+	    	    BG.trgPixmap = GrowPixmap(  BG.srcPixmap,
+						BG.Width, BG.Height,
 		                                width, height, gc, SHADING );
 		}
-		break ;	
-	case BGT_NoTileH : 
+		break ;
+	case BGT_NoTileH :
 		if( BG.finWidth != width )
 		{
-	    	    BG.trgPixmap = GrowPixmap(   BG.srcPixmap, 
-		    				 BG.Width, BG.Height, 
+	    	    BG.trgPixmap = GrowPixmap(   BG.srcPixmap,
+		    				 BG.Width, BG.Height,
 		                                 width, BG.Height, gc, SHADING );
 	    	    fin_height = BG.Height;
-		    
+
 		}
-		break ;	
+		break ;
 	case BGT_NoTileV :
 		if( BG.finHeight!=height )
 		{
-	    	    BG.trgPixmap = GrowPixmap(  BG.srcPixmap, 
-		    		                BG.Width, BG.Height, 
-		                    		BG.Width, height, gc, SHADING );			
-		    fin_width = BG.Width ; 
+	    	    BG.trgPixmap = GrowPixmap(  BG.srcPixmap,
+		    		                BG.Width, BG.Height,
+		                    		BG.Width, height, gc, SHADING );
+		    fin_width = BG.Width ;
 		}
-		break ;			
-	case BGT_Cut : 
+		break ;
+	case BGT_Cut :
 
 /*fprintf(stderr, "\n aterm RenderPixmap(): (BG_Cut)src pixmap is [%lu]",BG.srcPixmap); */
-		BG.trgPixmap = CutWinPixmap( TermWin.vt, BG.srcPixmap, 
-					  BG.Width, BG.Height, 
-		                          width, height, gc, 
+		BG.trgPixmap = CutWinPixmap( TermWin.vt, BG.srcPixmap,
+					  BG.Width, BG.Height,
+		                          width, height, gc,
 					  SHADING );
-		break ;		
+		break ;
 #ifdef _MYSTYLE_
 	case BGT_MyStyle :
-		BG.trgPixmap = RenderMyStylePixmap( BG.mystyle, BG.srcPixmap, 
-					            BG.Width, BG.Height, 
+		BG.trgPixmap = RenderMyStylePixmap( BG.mystyle, BG.srcPixmap,
+					            BG.Width, BG.Height,
 		                                    width, height );
 		if( BG.mystyle->texture_type == TEXTURE_PIXMAP)
 		{
@@ -782,19 +832,19 @@ RenderPixmap(int DontCheckSource )
     XFreeGC(Xdisplay, gc); /* don't need anymore */
     if( BG.trgPixmap != None )
     {
-	BG.finWidth = fin_width ; 
+	BG.finWidth = fin_width ;
 	BG.finHeight = fin_height;
 	XSync(Xdisplay, 0);
-        XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt, 
+        XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt,
     	                           BG.trgPixmap);
 	TermWin.LastPixmapUsed = BG.srcPixmap ;
-	BG.trgPixmapSet = 1 ;				   
+	BG.trgPixmapSet = 1 ;
 
 
 	if( BG.trgPixmap != BG.srcPixmap )
 	{/* don't need it anymore server has it */
-	    XFreePixmap( Xdisplay, BG.trgPixmap ); 
-	    XSync(Xdisplay, 0);	    	
+	    XFreePixmap( Xdisplay, BG.trgPixmap );
+	    XSync(Xdisplay, 0);
 	}
 	BG.trgPixmap = None ;
     }
@@ -805,7 +855,7 @@ RenderPixmap(int DontCheckSource )
 #endif      			   /* BACKGROUND_IMAGE || TRANSPARENT */
 
 
-#ifdef BACKGROUND_IMAGE 
+#ifdef BACKGROUND_IMAGE
 /* we need this stuff only to load background image from file */
 
 #ifdef USE_LIBASIMAGE
@@ -823,10 +873,10 @@ parse_pixmap_geom(const char *geom)
 
     if (geom == NULL)	return 0;
     if (!strlen(geom))	return 0;
-	
-    if (!strcmp(geom, "?")) 
+
+    if (!strcmp(geom, "?"))
     {
-      static char     str[] = "[10000x10000+10000+10000]";	/* should be big enough */    
+      static char     str[] = "[10000x10000+10000+10000]";	/* should be big enough */
 	sprintf(str, "[%dx%d+%d+%d]",
 		BG.srcWidth, BG.srcHeight, BG.srcX, BG.srcY );
 	xterm_seq(XTerm_title, str);
@@ -842,7 +892,7 @@ parse_pixmap_geom(const char *geom)
     MIN_IT(y, 10000);
     MIN_IT(w, 10000);
     MIN_IT(h, 10000);
-    	
+
     if( w != BG.srcWidth )
     {
         changed++;
@@ -876,20 +926,20 @@ LoadBGPixmap(const char *file)
     char           *realfilename;
     assert(file != NULL);
 
-    if (BG.srcPixmap != None && BG.bMySource ) 
+    if (BG.srcPixmap != None && BG.bMySource )
     {
 	XFreePixmap(Xdisplay, BG.srcPixmap);
 	BG.srcPixmap = None;
     }
-    
-    if (*file != '\0') 
+
+    if (*file != '\0')
     {
-        /* search environment variables here too */    
+        /* search environment variables here too */
 	if ((realfilename = (char *)File_find(file, ".xpm")) == NULL )
 	{
-#ifndef USE_LIBASIMAGE	
+#ifndef USE_LIBASIMAGE
 	    print_error("couldn't load XPM file [%s]", file);
-#endif	
+#endif
 	}else
 	{
 	    xpmAttr.closeness = 30000;
@@ -902,21 +952,21 @@ LoadBGPixmap(const char *file)
             if( (BG.srcPixmap = LoadImageWithMask( Xdisplay, Xroot, 256, realfilename, NULL )) !=
 	            None )
 	    {
-	      Window root; int x, y ; unsigned int bw, depth ;			
-	       if( XGetGeometry( Xdisplay, BG.srcPixmap, &root, &x, &y, 
+	      Window root; int x, y ; unsigned int bw, depth ;
+	       if( XGetGeometry( Xdisplay, BG.srcPixmap, &root, &x, &y,
 	                         &(xpmAttr.width), &(xpmAttr.height),&bw, &depth )==0)
 	       { xpmAttr.width = 0 ; xpmAttr.height = 0 ; }
 	    }
 #else
 	    if( XpmReadFileToPixmap(Xdisplay, Xroot, realfilename, &BG.srcPixmap, NULL, &xpmAttr) != 0 )
 	        print_error("couldn't load XPM file [%s]", realfilename);
-#endif  
+#endif
 	}
     }
-    
+
     /* need to add geometry processing code here */
-    
-    if( BG.srcPixmap == None )  
+
+    if( BG.srcPixmap == None )
     {
 	XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
 	BG.bMySource = 0 ;
@@ -925,7 +975,7 @@ LoadBGPixmap(const char *file)
 	BG.trgPixmapSet = 0 ;
     }
     else
-    { 
+    {
 	BG.bMySource = 1 ;
 	BG.Width = xpmAttr.width;
 	BG.Height = xpmAttr.height;
@@ -933,16 +983,16 @@ LoadBGPixmap(const char *file)
 	{
 	  int w, h;
           Pixmap tmp ;
-          
+
     	    if( BG.srcWidth <=0 ) w = BG.Width ;
     	    else w = min(BG.Width-BG.srcX,BG.srcWidth);
-	
+
     	    if( BG.srcHeight <=0 ) h = BG.Height ;
     	    else h = min(BG.Height-BG.srcY,BG.srcHeight);
 
     	    tmp = ShadePixmap(BG.srcPixmap, BG.srcX, BG.srcY, w, h,
 	                      DefaultGC(Xdisplay, Xscreen ), &(BG.Shading) );
-    	    if( tmp != None ) 
+    	    if( tmp != None )
 	    {
 	        XFreePixmap( Xdisplay, BG.srcPixmap );
 	        BG.srcPixmap = tmp ;
@@ -954,18 +1004,18 @@ LoadBGPixmap(const char *file)
 	{
 	  Pixmap tmp ;
           GC gc = DefaultGC(Xdisplay, Xscreen );
-	  
+
     	    tmp = ShadePixmap(BG.srcPixmap, 0, 0, BG.Width, BG.Height,
 			      DefaultGC(Xdisplay, Xscreen ), &(BG.Shading) );
-    	    if( tmp != None ) 
+    	    if( tmp != None )
 	    {
 	        XFreePixmap( Xdisplay, BG.srcPixmap );
 	        BG.srcPixmap = tmp ;
 	    }
 	}
-	RenderPixmap(1); 
+	RenderPixmap(1);
     }
-	
+
     scr_touch();
 }
-#endif				/* BACKGROUND_IMAGE */
\ No newline at end of file
+#endif				/* BACKGROUND_IMAGE */
