@@ -1,5 +1,5 @@
 /*
- * $Id: rxvt.h,v 1.10 2000/07/02 21:08:24 sashav Exp $
+ * $Id: rxvt.h,v 1.3 2001/03/26 17:26:10 sashav Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
@@ -47,7 +47,7 @@
 #ifdef _SCO_DS     /* SCO Osr5 - Hops */
     /* Scos pty naming is /dev/[pt]typ0 - /dev/[pt]ty256 */
 # define ALL_NUMERIC_PTYS
-#endif 
+#endif
 
 /*
  *****************************************************************************
@@ -143,6 +143,11 @@
 
 #include "libafterstep.h"
 
+#ifdef DEBUG_X
+#define XGetGeometry(dpy,win,r,x,y,w,h,b,d) \
+	trace_XGetGeometry(__FILE__,__LINE__,dpy,win,r,x,y,w,h,b,d)
+#endif
+
 #ifdef RXVT_GRAPHICS
 # include "rxvtgrx.h"
 #endif
@@ -194,7 +199,7 @@ typedef RUINTPT		R_u_int_p_t;
 
 typedef struct ShadingInfo{
     XColor      tintColor ;
-    unsigned int shading ;	
+    int shading ;
 }ShadingInfo ;
 
 #define NO_NEED_TO_SHADE(o) (o.shading==100 && o.tintColor.red==0xFFFF && o.tintColor.green==0xFFFF && o.tintColor.blue == 0xFFFF)
@@ -205,7 +210,7 @@ typedef struct _BackroundInfo {
     Pixmap	srcPixmap; /* after it was clipped out according to stuff below*/
     /* x,y,width and height of the subimage to be clipped out of
        the specifyed image and used as background */
-    struct MyStyle *mystyle ;   
+    struct MyStyle *mystyle ;
     unsigned int user_flags ;
     int 	srcWidth, srcHeight, srcX, srcY ;
     int 	Width, Height ; /* width and height of the loaded pixmap */
@@ -219,11 +224,11 @@ typedef struct _BackroundInfo {
 #define BGT_NO_TILE 	"notile"
 #define BGT_NO_TILE_H 	"notileh"
 #define BGT_NO_TILE_V 	"notilev"
-#define BGT_CUT     	"cut"			     
+#define BGT_CUT     	"cut"
 #define BGT_ALL      	"tile|center|scale|scaleh|scalev|notile|notileh|notilev|cut"
-    enum { BGT_Tile = 1, 
+    enum { BGT_Tile = 1,
 	   BGT_Center,
-	   BGT_Scale, 
+	   BGT_Scale,
 	   BGT_ScaleH,
 	   BGT_ScaleV,
 	   BGT_NoTile,
@@ -234,20 +239,20 @@ typedef struct _BackroundInfo {
 	   BGT_MyStyle,
 #endif
 	   BGT_None } trgType;
-	   
 
-    
+
+
     Pixmap      trgPixmap;
     int 	trgPixmapSet;
-#ifdef SCALING_GEOM_ENABLED    
+#ifdef SCALING_GEOM_ENABLED
     /* requested x,y,width and height of the target pixmap */
     int 	trgWidth, trgHeight, trgX, trgY ;
-#endif    
+#endif
     /* final width and height of constructed pixmap */
     int 	finWidth, finHeight ;
     int 	cutX, cutY ;
     ShadingInfo Shading ;
-    
+
 }BackgroundInfo ;
 #endif
 
@@ -470,9 +475,9 @@ typedef struct {
 # undef DEFINE_XTERM_COLOR	/* since kterm-color doesn't exist?   */
 #endif
 
-#ifdef ZH 
+#ifdef ZH
 # undef KANJI			/* can't put Chinese/Kanji together   */
-# undef GREEK_SUPPORT		
+# undef GREEK_SUPPORT
 # undef DEFINE_XTERM_COLOR
 #endif
 
@@ -491,7 +496,7 @@ typedef struct {
 # ifdef KANJI
 #  define TERMENV	"kterm"
 # else
-#  define TERMENV	"xterm"
+#  define TERMENV	"rxvt"
 # endif
 #endif
 
@@ -802,6 +807,10 @@ enum colour_list {
 #define Width2Pixel(n)		((n) * TermWin.fwidth)
 #define Height2Pixel(n)		((n) * TermWin.fheight)
 
+#ifdef THAI
+#define CharWidth(wf, ch)   ((wf)->per_char[(unsigned char)(ch) - (wf)->min_char_or_byte2].width)
+#endif
+
 #define TermWin_TotalWidth()	(TermWin.width  + 2 * TermWin_internalBorder)
 #define TermWin_TotalHeight()	(TermWin.height + 2 * TermWin_internalBorder)
 
@@ -965,7 +974,7 @@ EXTERN const char      *rs_shade;
 
 #ifdef OFF_FOCUS_FADING
 EXTERN const char      *rs_fade;
-#endif 
+#endif
 
 EXTERN const char      *rs_textType;
 /*
