@@ -1,8 +1,9 @@
 /*--------------------------------*-C-*--------------------------------------*
  * File:      screen.c
  *---------------------------------------------------------------------------*
- * Copyright (C) 2000 Sasha Vasko <sashav@sprintmail.com>
- * Copyright (C) 1997,1998 Geoff Wing <gcw@pobox.com>
+ * Copyright (C) 2001 (cursor color) Jordan DeLong <fracture at allusion dot net>
+ * Copyright (C) 2000 Sasha Vasko <sashav at sprintmail dot com>
+ * Copyright (C) 1997,1998 Geoff Wing <gcw at pobox dot com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,10 +34,12 @@
  */
 
 #ifndef lint
-static const char rcsid[] = "$Id: screen.c,v 1.5 2000/05/30 22:17:56 sashav Exp $";
+static const char rcsid[] = "$Id: screen.c,v 1.6 2001/08/09 05:25:05 sashav Exp $";
 #endif
 
 #define INTERN_SCREEN
+#define LIBAFTERSTEP_HEADER_INCLUDED
+
 #include "rxvt.h"		/* NECESSARY */
 
 #include <X11/Xatom.h>
@@ -139,47 +142,47 @@ ENC_METHOD encoding_method;
 
 # define drawBuffer	(TermWin.vt)
 
-void CLEAR_ROWS(unsigned row, unsigned num)						
-{ 
+void CLEAR_ROWS(unsigned row, unsigned num)
+{
   unsigned line_h = Height2Pixel(1)-1;
-  register unsigned hy = (line_h+1)*num, y = Row2Pixel(row);                          
-  
+  register unsigned hy = (line_h+1)*num, y = Row2Pixel(row);
+
     if(!TermWin.bMapped) return ;
 
-    if (y < TermWin_internalBorder+line_h) 
-    {y = 0; hy += TermWin_internalBorder;} 
-    if (y + hy > (TermWin.height+TermWin_internalBorder)-line_h) 
-	hy = (TermWin.height-y)+TermWin_internalBorders; 
-    XClearArea(Xdisplay, drawBuffer, 0, y, TermWin.width+TermWin_internalBorders, hy, 0);                    
+    if (y < TermWin_internalBorder+line_h)
+    {y = 0; hy += TermWin_internalBorder;}
+    if (y + hy > (TermWin.height+TermWin_internalBorder)-line_h)
+	hy = (TermWin.height-y)+TermWin_internalBorders;
+    XClearArea(Xdisplay, drawBuffer, 0, y, TermWin.width+TermWin_internalBorders, hy, 0);
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
-    if (TermWin.tintGC) 
-    {                                         
-        XFillRectangle(Xdisplay, drawBuffer, TermWin.tintGC,            
-                       0, y, TermWin.width+TermWin_internalBorders, hy);                              
-    }                                                               
+    if (TermWin.tintGC)
+    {
+        XFillRectangle(Xdisplay, drawBuffer, TermWin.tintGC,
+                       0, y, TermWin.width+TermWin_internalBorders, hy);
+    }
 #endif
 }
 
-void CLEAR_CHARS(unsigned x, unsigned y, unsigned num)						
-{ 
+void CLEAR_CHARS(unsigned x, unsigned y, unsigned num)
+{
   unsigned char_w = Width2Pixel(1)-1, line_h = Height2Pixel(1)-1;
-  register unsigned hx = (char_w+1)*num ,hy = line_h+1;                          
-    
+  register unsigned hx = (char_w+1)*num ,hy = line_h+1;
+
     if(!TermWin.bMapped) return ;
 
-    if (x < TermWin_internalBorder+char_w) {x = 0; hx += TermWin_internalBorder;} 
-    if (y < TermWin_internalBorder+line_h) {y = 0; hy += TermWin_internalBorder ;} 
+    if (x < TermWin_internalBorder+char_w) {x = 0; hx += TermWin_internalBorder;}
+    if (y < TermWin_internalBorder+line_h) {y = 0; hy += TermWin_internalBorder ;}
 
     if (x + hx > (TermWin.width+TermWin_internalBorder) -char_w)
-	hx = (TermWin.width+TermWin_internalBorders+1)-x; 
-    if (y + hy > (TermWin.height+TermWin_internalBorder) - line_h) 
-	hy = (TermWin.height+TermWin_internalBorders+1)-y; 
-    XClearArea(Xdisplay, drawBuffer, x, y, hx, hy, 0);                   
+	hx = (TermWin.width+TermWin_internalBorders+1)-x;
+    if (y + hy > (TermWin.height+TermWin_internalBorder) - line_h)
+	hy = (TermWin.height+TermWin_internalBorders+1)-y;
+    XClearArea(Xdisplay, drawBuffer, x, y, hx, hy, 0);
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
-    if (TermWin.tintGC) 
-    {                                         
+    if (TermWin.tintGC)
+    {
         XFillRectangle(Xdisplay, drawBuffer, TermWin.tintGC, x, y, hx, hy);
-    }                                                               
+    }
 #endif
 }
 
@@ -507,7 +510,7 @@ scr_poweron(void)
  * ------------------------------------------------------------------------- */
 /*
  * Save and Restore cursor
- * XTERM_SEQ: Save cursor   : ESC 7     
+ * XTERM_SEQ: Save cursor   : ESC 7
  * XTERM_SEQ: Restore cursor: ESC 8
  */
 /* PROTO */
@@ -973,6 +976,9 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 	    scr_insdel_chars(1, INSERT);
 	stp[screen.cur.col] = c;
 	srp[screen.cur.col] = rstyle;
+	/*
+         * TODO: This might end up wrapping the tone sign in Thai
+         */
 	if (screen.cur.col < (last_col - 1))
 	    screen.cur.col++;
 	else {
@@ -988,7 +994,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 
 /*
  * If we wrote anywhere in the selected area, kill the selection
- * XXX: should we kill the mark too?  Possibly, but maybe that 
+ * XXX: should we kill the mark too?  Possibly, but maybe that
  *      should be a similar check.
  */
     if (ROW_AND_COL_IS_BEFORE(screen.cur.row, screen.cur.col,
@@ -999,7 +1005,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 	i = 0;
     else
 	i = 1;
-    if (selection.op && current_screen == selection.screen 
+    if (selection.op && current_screen == selection.screen
 	&& wrotespecial != 0 && (i != wherecursor || i == 0))
 	CLEAR_SELECTION;
 
@@ -1378,7 +1384,6 @@ scr_insdel_chars(int count, int insdel)
 
     if (count <= 0)
 	return;
-
     CHECK_SELECTION(1);
     MIN_IT(count, (TermWin.bcol - screen.cur.col));
 
@@ -1600,9 +1605,8 @@ scr_report_position(void)
     tt_printf((unsigned char *) "\033[%d;%dR",
 	      screen.cur.row + 1, screen.cur.col + 1);
 }
-
 /* ------------------------------------------------------------------------- *
- *                                  FONTS                                    * 
+ *                                  FONTS                                    *
  * ------------------------------------------------------------------------- */
 
 /*
@@ -1671,7 +1675,7 @@ scr_charset_set(int set, unsigned int ch)
 }
 
 /* ------------------------------------------------------------------------- *
- *          MULTIPLE-CHARACTER FONT SET MANIPULATION FUNCTIONS               * 
+ *          MULTIPLE-CHARACTER FONT SET MANIPULATION FUNCTIONS               *
  * ------------------------------------------------------------------------- */
 #ifdef MULTICHAR_SET
 # ifdef KANJI
@@ -1737,7 +1741,7 @@ set_multichar_encoding(const char *str)
 #endif				/* MULTICHAR_SET */
 
 /* ------------------------------------------------------------------------- *
- *                           GRAPHICS COLOURS                                * 
+ *                           GRAPHICS COLOURS                                *
  * ------------------------------------------------------------------------- */
 
 #ifdef RXVT_GRAPHICS
@@ -1758,7 +1762,7 @@ scr_get_bgcolor(void)
 #endif
 
 /* ------------------------------------------------------------------------- *
- *                        MAJOR SCREEN MANIPULATION                          * 
+ *                        MAJOR SCREEN MANIPULATION                          *
  * ------------------------------------------------------------------------- */
 
 /*
@@ -1984,7 +1988,7 @@ scr_refresh(int type)
 		    morecur = 0,
                     xpixel,	/* x offset for start of drawing (font)      */
                     ypixel;	/* y offset for start of drawing (font)      */
-		    
+
     static int      focus = -1;	/* screen in focus?                          */
     long            gcmask;	/* Graphics Context mask                     */
     unsigned long   ltmp;
@@ -2009,6 +2013,11 @@ scr_refresh(int type)
     static row_col_t oldcursor = { -1, -1};
     				/* is there an old outline cursor on screen? */
 
+#ifdef THAI  /* variables to clean un-touched space at the end of screen */
+    int		    thaicol_stp_offset = 0;
+    int		    thaicol_dtp_offset = 0;
+#endif
+
 #ifndef NO_BOLDFONT
     int             bfont = 0;	/* we've changed font to bold font           */
 #endif
@@ -2092,7 +2101,9 @@ scr_refresh(int type)
 
 		if (do_stuff)
 		{
+#ifndef THAI    /* this mess up Thai chars, we have to implement ourselves */
 		    dtp[col + 1] = 0;
+#endif
 # if defined(MULTICHAR_SET) && ! defined(NO_BOLDOVERSTRIKE_MULTI)
 		    if ((srp[col] & RS_multiMask) == RS_multi2) {
 		        col--;
@@ -2125,41 +2136,48 @@ scr_refresh(int type)
     currow = screen.cur.row + TermWin.saveLines;
     if (focus != TermWin.focus)
 	focus = TermWin.focus;
-    if (screen.flags & Screen_VisibleCursor && focus) {
-	srp = &(screen.rend[currow][screen.cur.col]);
-	*srp ^= RS_RVid;
+    if (screen.flags & Screen_VisibleCursor && focus)
+	{
+		srp = &(screen.rend[currow][screen.cur.col]);
+#ifdef THAI
+		if(CharWidth(wf,screen.text[currow][screen.cur.col])<=0)
+			*srp ^= RS_Bold;
+#endif
 #ifndef NO_CURSORCOLOR
-	cc1 = *srp & (RS_fgMask | RS_bgMask);
-	if (Xdepth <= 2 || !rs_color[Color_cursor])
-	    ccol1 = Color_fg;
-	else
-	    ccol1 = Color_cursor;
-	if (Xdepth <= 2 || !rs_color[Color_cursor2])
-	    ccol2 = Color_bg;
-	else
-	    ccol2 = Color_cursor2;
-	*srp = SET_FGCOLOR(*srp, ccol1) | SET_BGCOLOR(*srp, ccol2);
+		cc1 = *srp & (RS_fgMask | RS_bgMask);
+		if (Xdepth <= 2 || !rs_color[Color_cursor])
+	    	ccol1 = Color_fg;
+		else
+	    	ccol1 = Color_cursor;
+		if (Xdepth <= 2 || !rs_color[Color_cursor2])
+	    	ccol2 = Color_bg;
+		else
+	    	ccol2 = Color_cursor2;
+	/*  changed by rev vide cursor patch */
+		*srp = SET_BGCOLOR(*srp, ccol1);
+		*srp = SET_FGCOLOR(*srp, ccol2);
 #endif
 #ifdef MULTICHAR_SET
-	rt1 = *srp & RS_multiMask;
-	if (rt1 == RS_multi1) {
-	    if (screen.cur.col < TermWin.bcol - 2
-		&& ((srp[1] & RS_multiMask) == RS_multi2))
-		morecur = 1;
-	} else if (rt1 == RS_multi2) {
-	    if (screen.cur.col > 0
-		&& ((srp[-1] & RS_multiMask) == RS_multi1))
-		morecur = -1;
-	}
-	if (morecur) {
-	    srp += morecur;
-	    *srp ^= RS_RVid;
-	}
+		rt1 = *srp & RS_multiMask;
+		if (rt1 == RS_multi1) {
+	    	if (screen.cur.col < TermWin.bcol - 2
+			&& ((srp[1] & RS_multiMask) == RS_multi2))
+			morecur = 1;
+		} else if (rt1 == RS_multi2) {
+	    	if (screen.cur.col > 0
+			&& ((srp[-1] & RS_multiMask) == RS_multi1))
+			morecur = -1;
+		}
+		if (morecur) {
+	    	srp += morecur;
+	    	*srp ^= RS_RVid;
+		}
 # ifndef NO_CURSORCOLOR
-	if (morecur) {
-	    cc2 = *srp & (RS_fgMask | RS_bgMask);
-	    *srp = SET_FGCOLOR(*srp, ccol1) | SET_BGCOLOR(*srp, ccol2);
-	}
+		if (morecur) {
+	    	cc2 = *srp & (RS_fgMask | RS_bgMask);
+	    	*srp = SET_BGCOLOR(*srp, ccol1);
+	    	*srp = SET_FGCOLOR(*srp, ccol2);
+		}
 # endif
 #endif
     }
@@ -2197,6 +2215,30 @@ scr_refresh(int type)
  * E: OK, now the real pass
  */
     for (row = 0; row < TermWin.nrow; row++) {
+#ifdef THAI
+        char thai_update[TermWin.ncol];    /* update map */
+	int stpx, dtpx;
+	/* compare drawn_text and screen.text and check which to update */
+	ThaiUpdateMap(screen.text[row+row_offset],drawn_text[row],
+		      screen.rend[row+row_offset],drawn_rend[row],
+ 		      thai_update,TermWin.ncol);
+
+/*
+ * TODO: Change this algo to scalefont compatible
+ */
+        thaicol_stp_offset = 0;  /* records how many column deficit */
+        thaicol_dtp_offset = TermWin.ncol - Thai_ColMaxPaint(dtp,TermWin.ncol);
+        /* clean strings before redraw */
+        for(col = 0; col < TermWin.ncol; col++){
+            if(!ThaiIsMiddleLineCh(stp[col])) thaicol_stp_offset++;
+        }
+
+        if(thaicol_stp_offset > thaicol_dtp_offset){
+          CLEAR_CHARS(Col2Pixel( TermWin.ncol - thaicol_stp_offset ),
+                      ypixel - TermWin.font->ascent, thaicol_stp_offset);
+
+        }
+#endif
 	scrrow = row + row_offset;
 	stp = screen.text[scrrow];
 	srp = screen.rend[scrrow];
@@ -2206,10 +2248,14 @@ scr_refresh(int type)
 	/* compare new text with old - if exactly the same then continue */
 	    rt1 = srp[col];	/* screen rendition */
 	    rt2 = drp[col];	/* drawn rendition  */
+#ifdef THAI  /* updating conditional */
+	    if (!thai_update[col]) {
+#else
 	    if ((stp[col] == dtp[col])	/* must match characters to skip */
 		&& ((rt1 == rt2)	/* either rendition the same or  */
 		    || ((stp[col] == ' ')	/* space w/ no bg change */
 			&& (GET_BGATTR(rt1) == GET_BGATTR(rt2))))) {
+#endif
 #ifdef MULTICHAR_SET
 	    /* if first byte is Kanji then compare second bytes */
 		if ((rt1 & RS_multiMask) != RS_multi1)
@@ -2224,13 +2270,21 @@ scr_refresh(int type)
 #endif
 	    }
 	/* redraw one or more characters */
+#ifndef THAI  /* can not set dtp as we go because it causes
+               * unpredictable results (bug)
+               */
 	    dtp[col] = stp[col];
+#endif
 	    rend = drp[col] = srp[col];
 
 	    len = 0;
 	    buffer[len++] = stp[col];
 	    ypixel = TermWin.font->ascent + Row2Pixel(row);
+#ifdef THAI   /* compute x coordinate */
+	    xpixel = ThaiCol2Pixel( col, screen.text[scrrow]);
+#else
 	    xpixel = Col2Pixel(col);
+#endif /* THAI */
 	    wlen = 1;
 
 /*
@@ -2386,7 +2440,11 @@ scr_refresh(int type)
  * Actually do the drawing of the string here
  */
 	    if (fprop) {
+#ifdef THAI
+		if (rvid && CharWidth(wf,stp[col])>0) {
+#else
 		if (rvid) {
+#endif
 		    SWAP_IT(gcvalue.foreground, gcvalue.background, ltmp);
 		    gcmask |= (GCForeground | GCBackground);
 		    XChangeGC(Xdisplay, TermWin.gc, gcmask, &gcvalue);
@@ -2396,8 +2454,17 @@ scr_refresh(int type)
 		    SWAP_IT(gcvalue.foreground, gcvalue.background, ltmp);
 		    XChangeGC(Xdisplay, TermWin.gc, gcmask, &gcvalue);
 		} else
+#ifdef THAI
+ 		   if(CharWidth(wf,stp[col])>0)
+#endif
 		    CLEAR_CHARS(xpixel, ypixel - TermWin.font->ascent, 1);
 		DRAW_STRING(draw_string, xpixel, ypixel, buffer, 1);
+#ifdef THAI
+/*
+ * Trap error
+ */
+ 		      if(wlen > 1) fprintf(stderr, "wlen > 1: %d\n", wlen);
+#endif
 #ifndef NO_BOLDOVERSTRIKE
 		if (MONO_BOLD(rend))
 		    DRAW_STRING(draw_string, xpixel + 1, ypixel, buffer, 1);
@@ -2405,6 +2472,9 @@ scr_refresh(int type)
 	    } else
 #ifdef TRANSPARENT
 	    if ((Options & Opt_transparent) && back == Color_bg) {
+#ifdef THAI
+	        if(CharWidth(wf,stp[col])>0)
+#endif
 		CLEAR_CHARS(xpixel, ypixel - TermWin.font->ascent, len);
 		DRAW_STRING(draw_string, xpixel, ypixel, buffer, wlen);
 	    } else
@@ -2435,23 +2505,46 @@ scr_refresh(int type)
 	    }
 	}
     }
+#ifdef THAI
+/*
+ * Hack: update drawn_text
+ */
+	for (row = 0; row < TermWin.nrow; row++)
+	{
+		scrrow = row + row_offset;
+		stp = screen.text[scrrow];
+		srp = screen.rend[scrrow];
+		dtp = drawn_text[row];
+		drp = drawn_rend[row];
+
+		for(col=0 ; col<TermWin.ncol ; col++)
+		{
+			dtp[col] = stp[col];
+			drp[col] = srp[col];
+		}
+	}
+#endif
 
 /*
  * F: cleanup cursor and display outline cursor in necessary
  */
-    if (screen.flags & Screen_VisibleCursor) {
-	if (focus) {
-	    srp = &(screen.rend[currow][screen.cur.col]);
-	    *srp ^= RS_RVid;
+    if (screen.flags & Screen_VisibleCursor)
+	{
+		if (focus)
+		{
+	    	srp = &(screen.rend[currow][screen.cur.col]);
+#ifdef THAI
+	    	if (CharWidth(wf,screen.text[currow][screen.cur.col])<=0)
+	        	*srp ^= RS_Bold;
+#endif
 #ifndef NO_CURSORCOLOR
-	    *srp = (*srp & ~(RS_fgMask | RS_bgMask)) | cc1;
+	    	*srp = (*srp & ~(RS_fgMask | RS_bgMask)) | cc1;
+#endif
+	    	if (morecur) {
+				srp += morecur;
+#if defined(MULTICHAR_SET) && ! defined(NO_CURSORCOLOR)
+			*srp = (*srp & ~(RS_fgMask | RS_bgMask)) | cc2;
 #endif
-	    if (morecur) {
-		srp += morecur;
-		*srp ^= RS_RVid;
-# if defined(MULTICHAR_SET) && ! defined(NO_CURSORCOLOR)
-		*srp = (*srp & ~(RS_fgMask | RS_bgMask)) | cc2;
-# endif
 	    }
 	} else {
 	    currow = screen.cur.row - TermWin.view_start;
@@ -2467,10 +2560,19 @@ scr_refresh(int type)
 		    gcvalue.foreground = PixColors[Color_fg];
 		}
 #endif
+#ifdef THAI
+		XDrawRectangle(Xdisplay, drawBuffer, TermWin.gc,
+			       ThaiCol2Pixel(col,screen.text[screen.cur.row+TermWin.saveLines]),
+			       Row2Pixel(currow),
+			       Width2Pixel(1 + wbyte) - 1,
+			       Height2Pixel(1) - 1);
+
+#else
 		XDrawRectangle(Xdisplay, drawBuffer, TermWin.gc,
 			       Col2Pixel(col), Row2Pixel(currow),
 			       Width2Pixel(1 + wbyte) - 1,
 			       Height2Pixel(1) - 1);
+#endif
 #ifndef NO_CURSORCOLOR
 		if (gcmask)	/* restore normal colours */
 		    XChangeGC(Xdisplay, TermWin.gc, gcmask, &gcvalue);
@@ -2479,6 +2581,7 @@ scr_refresh(int type)
 	}
     }
 
+
 /*
  * G: cleanup selection
  */
@@ -2500,11 +2603,11 @@ scr_clear_tint(int bWithTinting)
 {
     if( !TermWin.bMapped ) return ;
 #ifdef TRANSPARENT
-    if (Options & Opt_transparent) 
+    if (Options & Opt_transparent)
     {
 /*	int n ;
 	fprintf( stderr, "\n Clearing Parents ...");
-	for( n = 0 ; n < ParentWinNum ; n++ ) 
+	for( n = 0 ; n < ParentWinNum ; n++ )
 */
         XClearWindow(Xdisplay, TermWin.parent);
 /*	XFlush( Xdisplay );
@@ -2513,7 +2616,7 @@ scr_clear_tint(int bWithTinting)
 */    }
 #endif
     XClearWindow(Xdisplay, TermWin.vt);
-    
+
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
     if (TermWin.tintGC && bWithTinting)
     {
@@ -2526,7 +2629,7 @@ scr_clear_tint(int bWithTinting)
 /* ------------------------------------------------------------------------- */
 /* colors has changed so we have to repaint self			     */
 /* PROTO */
-void 
+void
 on_colors_changed(int idx)
 {
     colors_changed = 1 ;
@@ -2578,7 +2681,7 @@ scr_reverse_selection(void)
 }
 
 /* ------------------------------------------------------------------------- *
- *                           CHARACTER SELECTION                             * 
+ *                           CHARACTER SELECTION                             *
  * ------------------------------------------------------------------------- */
 
 /*
@@ -2597,7 +2700,7 @@ selection_check(int check_more)
 	|| (selection.end.row < -TermWin.nscrolled)
 	|| (selection.end.row >= TermWin.nrow))
 	CLEAR_ALL_SELECTION;
-    
+
     if (check_more == 1 && current_screen == selection.screen) {
     /* check for cursor position */
 	pos.row = screen.cur.row;
@@ -2718,7 +2821,7 @@ selection_clear(void)
 }
 
 /* ------------------------------------------------------------------------- */
-/* 
+/*
  * Copy a selection into the cut buffer
  * EXT: button 1 or 3 release
  */
@@ -2816,8 +2919,11 @@ selection_click(int clicks, int x, int y)
 
     clicks = ((clicks - 1) % 3) + 1;
     selection.clicks = clicks;	/* save clicks so extend will work */
-
+#ifdef THAI
+    selection_start_colrow(ThaiPixel2Col(x,y), Pixel2Row(y));
+#else
     selection_start_colrow(Pixel2Col(x), Pixel2Row(y));
+#endif
     if (clicks == 2 || clicks == 3)
 	selection_extend_colrow(selection.mark.col,
 				selection.mark.row + TermWin.view_start,
@@ -2957,7 +3063,11 @@ selection_extend(int x, int y, int flag)
 {
     int             col, row;
 
+#ifdef THAI
+    col = ThaiPixel2Col( x, y);
+#else
     col = Pixel2Col(x);
+#endif
     row = Pixel2Row(y);
     MAX_IT(row, 0);
     MIN_IT(row, TermWin.nrow - 1);
@@ -3085,7 +3195,7 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
  *     the selection we are closest to.
  *  c) button3 drag : extend/contract character/word/line - we select around
  *     a point/word/line which is either the start or end of the selection
- *     and it was decided by whichever point/word/line was `fixed' at the 
+ *     and it was decided by whichever point/word/line was `fixed' at the
  *     time of the most recent button3 press
  */
     if (button3 && buttonpress) {	/* button3 press */
@@ -3194,7 +3304,11 @@ void
 selection_rotate(int x, int y)
 {
     selection.clicks = selection.clicks % 3 + 1;
+#ifdef THAI
+    selection_extend_colrow(ThaiPixel2Col(x,y), Pixel2Row(y), 1, 0, 1);
+#else
     selection_extend_colrow(Pixel2Col(x), Pixel2Row(y), 1, 0, 1);
+#endif
 }
 
 /* ------------------------------------------------------------------------- */
@@ -3244,9 +3358,9 @@ selection_send(XSelectionRequestEvent * rq)
     }
     XSendEvent(Xdisplay, rq->requestor, False, 0, &ev);
 }
-
+
 /* ------------------------------------------------------------------------- *
- *                              MOUSE ROUTINES                               * 
+ *                              MOUSE ROUTINES                               *
  * ------------------------------------------------------------------------- */
 
 /*
@@ -3256,10 +3370,14 @@ selection_send(XSelectionRequestEvent * rq)
 void
 pixel_position(int *x, int *y)
 {
-    *x = Pixel2Col(*x);
-/* MAX_IT(*x, 0); MIN_IT(*x, TermWin.bcol - 1); */
-    *y = Pixel2Row(*y);
-/* MAX_IT(*y, 0); MIN_IT(*y, TermWin.nrow - 1); */
+#ifdef THAI
+  *x = ThaiPixel2Col(*x, *y);
+#else
+  *x = Pixel2Col(*x);
+#endif
+  /* MAX_IT(*x, 0); MIN_IT(*x, TermWin.bcol - 1); */
+  *y = Pixel2Row(*y);
+  /* MAX_IT(*y, 0); MIN_IT(*y, TermWin.nrow - 1); */
 }
 
 /* ------------------------------------------------------------------------- */
@@ -3272,7 +3390,7 @@ mouse_tracking(int report, int x, int y, int firstrow, int lastrow)
 }
 
 /* ------------------------------------------------------------------------- *
- *                              DEBUG ROUTINES                               * 
+ *                              DEBUG ROUTINES                               *
  * ------------------------------------------------------------------------- */
 /* ARGSUSED */
 /* PROTO */
