@@ -192,8 +192,6 @@ typedef struct {
 }IconGC;
 
 
-
-
 static void 
 init_scroll_size(void)
 {
@@ -326,7 +324,7 @@ renderIcon(char **data, Icon* pIcon, IconGC* igc)
 }
 
 static void
-PlaceIcon( Icon* i, int x, int y, Pixmap buffer )
+PlaceIcon( Icon* i, int x, int y, Drawable buffer )
 {
 #ifdef TRANSPARENT
     if( IS_TRANSP_SCROLL ) 
@@ -495,7 +493,7 @@ Pixmap scrollbar_fill_back( unsigned int height, int check_cache )
     {
       Pixmap root_pmap = None ;
       unsigned int root_width = 0, root_height = 0 ;
-	if( check_cache )
+	if( check_cache == 1 )
 	{
 	  int cache_valid = 0 ;
     	    root_pmap = ValidatePixmap(root_pmap, 1, 1, &root_width, &root_height);
@@ -504,19 +502,19 @@ Pixmap scrollbar_fill_back( unsigned int height, int check_cache )
 	      int my_x, my_y ;
 		if( GetWinPosition(scrollBar.win, &my_x, &my_y) )
 		{
-		    if( my_x== tCache.x || my_y == tCache.y ) cache_valid = 1 ;
+		    if( my_x== tCache.x && my_y == tCache.y ) cache_valid = 1 ;
 		    else { tCache.x = my_x ; tCache.y = my_y ; }
 		}else cache_valid = 1 ;
 	    }else { tCache.root = root_pmap ; tCache.height = height ; }		
-	    if( !cache_valid )
-	    {
+	    if( cache_valid == 0 )
+	    { 
 		if(tCache.cache) XFreePixmap( Xdisplay, tCache.cache );
 		tCache.cache = CutWinPixmap( scrollBar.win, root_pmap, root_width, root_height, SB_WIDTH+SB_BORDER_WIDTH, height, grayGC, &(TermWin.background.Shading));
 	    }
 	}
 	if( tCache.cache != None )
 	{    
-	    XCopyArea( Xdisplay, tCache.cache, buffer, grayGC, 0, 0, SB_WIDTH+SB_BORDER_WIDTH, height, 0, 0 );    
+	    FillPixmapWithTile( buffer, tCache.cache, 0, 0, SB_WIDTH+SB_BORDER_WIDTH, height, 0, 0 );    
 	    if( TermWin.tintGC)
     		XFillRectangle(Xdisplay, buffer, TermWin.tintGC, 0, 0, SB_WIDTH+SB_BORDER_WIDTH-1, height-1);
 	    return buffer ;
