@@ -35,10 +35,9 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: xdefaults.c,v 1.5 2000/05/30 22:17:56 sashav Exp $";
+static const char rcsid[] = "$Id: xdefaults.c,v 1.3 2001/08/09 05:25:05 sashav Exp $";
 #endif
 
-#include "libafterstep.h"
 #include "rxvt.h"		/* NECESSARY */
 
 /* #define DEBUG_RESOURCES */
@@ -140,7 +139,7 @@ static const struct {
     STRG(rs_geometry, NULL, "g", NULL, NULL),	/* short form */
     BOOL(rs_reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
          "reverse video"),
-#ifdef _MYSTYLE_	 
+#ifdef _MYSTYLE_
     STRG(rs_mystyle, "MyStyle", "mst", "name",
          "MyStyle"),
 #endif
@@ -189,19 +188,19 @@ static const struct {
          "bgtype", BGT_ALL, "type of the background pixmap transformation"),
     STRG(rs_tintType,
          "tintingType",
-         "tinttype", TINT_TYPE_ALL, 
+         "tinttype", TINT_TYPE_ALL,
 	 "defines function to be used for background tinting"),
     STRG(rs_shade,
          "shading",
          "sh", "%", "make transparent background x% darker"),
     STRG(rs_color[Color_tint],
-         "tinting", 
+         "tinting",
 	 "tint", "color",
          "tinted transparency color"),
 #endif
     STRG(rs_textType,
          "textType",
-         "txttype", GC_TYPE_ALL, 
+         "txttype", GC_TYPE_ALL,
 	 "defines function to be used for text drawing"),
 #ifdef OFF_FOCUS_FADING
     STRG(rs_fade,
@@ -394,6 +393,9 @@ list_options()
 #ifdef ZH
     fprintf(stderr, "Chinese,");
 #endif
+#ifdef THAI
+    fprintf(stderr, "Thai,");
+#endif
 #ifdef XTERM_SCROLLBAR
     fprintf(stderr, "XTerm-scrollbar,");
 #endif
@@ -408,12 +410,12 @@ list_options()
 #endif
 #ifdef TRANSPARENT
     fprintf(stderr, "transparency,");
-#else    
+#else
     fprintf(stderr, "no transparency,");
 # endif
 #ifdef OFF_FOCUS_FADING
     fprintf(stderr, "fading,");
-#else    
+#else
     fprintf(stderr, "no fading,");
 # endif
 #ifdef NEXT_SCROLLBAR
